#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django import template
from django.template import Context, loader
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response

from django.contrib.auth.models import User
from django.contrib.auth import authenticate, login
from django.views.generic.list_detail import object_list

from moapps.events.models import Event
from moapps.contacts.models import Town

from models import Profile
from htmllib import HTMLParser
from formatter import NullFormatter
from django.conf import settings
import sys
import urllib, httplib2


def index(request, template_name):
    rc = template.RequestContext(request)
    allstars = Profile._default_manager.all().filter( member = True ).order_by( '-karma' )[:5]
    members  = Profile._default_manager.all().filter( member = True ).order_by( '-karma' )[5:]
    return render_to_response(template_name, { 'stars': allstars, 'members' : members }, context_instance=rc)


def show_profile(request, template_name):
    rc = template.RequestContext(request)
    user    = rc['user']
    profile = user.get_profile()
    
    if profile.autoupdate:
        update_launchpad_user( user, profile )
        profile.autoupdate = False
        user.save()
        profile.save()
    
    tasks         = get_project_tasks( owner = user )
    nottasks      = get_project_tasks( owner = None )

    return render_to_response(template_name, {
        'profile'     : profile,
        'tasks'       : tasks,
        'nottasks'    : nottasks,
    }, context_instance=rc)


def get_project_tasks( owner = None ):
    list = [] 
    result = []
    for item in list:
        result.append( {
            'id'     : item.id,
            'typeid' : item.project.id,
            'type'   : 'projects',
            'name'   : item.name,
        } )
    return result



def join_loco(request):
    rc = template.RequestContext(request)
    profile = rc['user'].get_profile()
    profile.autoupdate = True
    profile.save()
    return HttpResponseRedirect('https://launchpad.net/~ubuntu-us-ma/+join') 


def update_profile(request, template_name):
    rc = template.RequestContext(request)
    user    = rc['user']
    profile = user.get_profile()
    update_launchpad_user( user, profile )
    user.save()
    profile.save()
    return HttpResponseRedirect(request.build_absolute_uri('/accounts/profile/'))


def create_profile(user, request):
    town = get_object_or_404(Town, pk=request.POST['town'])
    profile    = Profile( user, None, town, lpaccount, 0 )
    profile.save()


def login_or_register(request, data):
    # Look for existing user
    username = data['username'].replace('-', '_')
    try:
        user = User.objects.get( username=username )
        profile = user.get_profile()
        if profile.openid != data['id']:
            # Mismatch of user data
            return render_to_response('accounts/error.html', { 'error': 'Launchpad OpenID has Changed, possibly broken.' })
    except User.DoesNotExist:
        # Register new user account for this launchpad person
        user = User.objects.create_user(username, "%s@launchpad.net" % data['id'], data['password'])
        user.is_active = True
        # Store username seperatly because they can be different
        profile = Profile(
            user=user,
            username=data['username'],
            openid=data['id'],
            title=None,
            karma=0,
            imageurl=None,
        )
        update_launchpad_user(user, profile)
        user.save()
        profile.save()
    except Profile.DoesNotExist:
        return render_to_response('accounts/error.html', { 'error': 'User exists but profile does not, broken user.' })

    # Logon user by authenticating and then adding to session
    user = authenticate(username=username, password=data['password'])
    if user is not None:
        if user.is_active:
            login(request, user)
        else:
            return render_to_response('accounts/error.html', { 'error': 'Account is Disabled' })
    else:
        return render_to_response('accounts/error.html', { 'error': 'Invalid Login Details' })

    return HttpResponseRedirect(request.build_absolute_uri('/accounts/profile/'))




def update_launchpad_user(user, profile):
    username = profile.username
    content = launchpad_http_request( 'https://launchpad.net/~%s' % username )
    profile.member = False

    class LaunchpadParser(HTMLParser):
        lpmode = None

        def proc_fucked_up_attrs(self, attrs):
            attr = {}
            for a in attrs:
                attr[a[0]] = a[1]
            return attr

        def do_img(self, attrs):
            attr = self.proc_fucked_up_attrs(attrs)
            if attr.has_key('src'):
                if attr['src'][0] == '/':
                    attr['src'] = 'http://launchpad.net' + attr['src']
            if attr.has_key('width') and attr.has_key('height'):
                if attr['width'] == '192' and attr['height'] == '192':
                    profile.imageurl = attr['src']
                    #sys.stderr.write( "Photo -> " )
            if attr.has_key('title') and attr['title'] == 'Icon of ubuntu-us-ma':
                profile.member = True
            #sys.stderr.write( "Found image: %s\n" % str(attr) )
            return HTMLParser.do_img(self, attrs)

        def start_title(self, attrs):
            self.lpmode = 'title'
            return HTMLParser.start_title(self, attrs)

        def handle_data(self, data):
            data = data.strip()
            if not data:
                return
            if self.lpmode == 'title':
                name = data.split(' in Launchpad')
                if name[0]:
                    user.first_name = name[0].split(' ')[0]
                    user.last_name = ' '.join(name[0].split(' ')[1:])
            elif self.lpmode == 'email':
                user.email = data

            self.lpmode = None

            if data == 'Email:':
                self.lpmode = 'email'

            #sys.stderr.write("Data is %s\n" % data)
            return HTMLParser.handle_data(self, data)

    if content:
        p = LaunchpadParser(NullFormatter())
        p.feed(str(content))
        p.close()
    else:
        sys.stderr.write("No content came back!! Oh dear!")



def launchpad_http_request( url ):
    dn = url.split('/')[2]
    tried = 0
    result = {}

    if not settings.CACHE.has_key('sessions'):
        settings.CACHE['sessions'] = {}
    if not settings.CACHE['sessions'].has_key(dn):
        settings.CACHE['sessions'][dn] = launchpad_login( settings.LAUNCHPAD_EMAIL, settings.LAUNCHPAD_PASSWORD )
        settings.CACHE.save()
        tried = 1
    cookie = settings.CACHE['sessions'][dn]

    http = httplib2.Http()
    response, content = http.request( url, 'GET', headers={ 'Cookie' : str(cookie) } )

    if content.find('Log In') > 0:
        del(settings.CACHE['sessions'][dn])
        if tried:
            sys.stderr.write("Couldn't get user details, launchpad user is not logged in.")
            return ''
        return http_request( url )

    return content


def launchpad_login( email, password ):
    http = httplib2.Http()
    headers = {}

    # Get session header (cookies)
    response, content = http.request('https://launchpad.net/', 'GET')
    import sys
    sys.stderr.write("Headers: %s\n" % response)
    #headers['Cookie'] = response['set-cookie']

    # Login to session
    details = { 'loginpage_password' : password, 'loginpage_email' : email, 'loginpage_submit_login' : 'Log In' }
    response, content = http.request('https://launchpad.net/+login', 'POST', headers=headers, body=urllib.urlencode(details))

    # Confirm Logged on
    if content.find('Log In') > 0:
        sys.stderr.write("Not logged into launchpad!")
    else:
        sys.stderr.write("Logged back into launchpad!")
    return #headers['Cookie']

