from django.shortcuts import render_to_response as render
from django.http      import HttpResponse, HttpResponseRedirect
from django.conf      import settings
from django           import template
from UbuntuMaWebsite.accounts.views import login_or_register

from openid.consumer          import consumer
from openid.consumer.discover import OpenIDServiceEndpoint, OPENID_2_0_TYPE
from openid.yadis.discover    import DiscoveryFailure
from openid.fetchers          import HTTPFetchingError
from openid.message           import IDENTIFIER_SELECT
from openid.extensions.sreg   import SRegRequest, SRegResponse

from store import LaunchpadOpenIDStore
import crypt, sys


def openid_login(request, template_name=None):
    rc = template.RequestContext(request)
    cons = consumer.Consumer(request.session, LaunchpadOpenIDStore())
    server   = OpenIDServiceEndpoint()
    server.type_uris  = OPENID_2_0_TYPE
    server.server_url = 'https://login.launchpad.net/+openid'
    server.claimed_id = 'http://specs.openid.net/auth/2.0/identifier_select'
    try:
        req = cons.beginWithoutDiscovery(server)
    except HTTPFetchingError:
        ct = { 'openid_result': 'Failed to resolve OpenID.' }
        return render(template_name, ct, context_instance=rc)
    except DiscoveryFailure, e:
        ct = { 'openid_result': e.message }
        return render(template_name, ct, context_instance=rc)
    else:
        if req is None:
            ct = { 'openid_result': "Launchpad doesn't seem to respond!" }
            return render(template_name, ct, context_instance=rc)

    return_to = request.build_absolute_uri('/accounts/openid/done/')
    # Oops. Django doesn't use a None value for default so you cannot pass ''
    trust_root = request.build_absolute_uri('/***')[:-3]

    req.addExtension(SRegRequest(required=['nickname']))

    if req.shouldSendRedirect():
        redirect = req.redirectURL(trust_root, return_to)
        return HttpResponseRedirect(redirect)
    else:
        form = req.formMarkup(trust_root, return_to, form_tag_attrs={'id': 'openid_message'})
        ct = { 'openidform' : form }
        return render(template_name, ct, context_instance=rc)


def openid_return(request, template_name=None):
    rc = template.RequestContext(request)
    cons = consumer.Consumer(request.session, LaunchpadOpenIDStore())
    query = {}
    for item in request.GET:
        query[item] = request.GET[item]
    for item in request.POST:
        query[item] = request.POST[item]
    return_to = request.build_absolute_uri('/accounts/openid/done/')
    info = cons.complete(query, current_url=return_to)
    if info.status == consumer.FAILURE:
        ct = { 'openid_result': 'OpenID error: %s.' % info.message }
        return render(template_name, ct, context_instance=rc)
    if info.status == consumer.CANCEL:
        ct = { 'openid_result': 'Verification canceled.' }
        return render(template_name, ct, context_instance=rc)
    if info.status != consumer.SUCCESS:
        ct = { 'openid_result': 'Unknown status.' }
        return render(template_name, ct, context_instance=rc)
    # success!

    sreg_resp = SRegResponse.fromSuccessResponse(info)
    id = query['openid.claimed_id'].split('/')[-1]
    userdata = { 'username' : sreg_resp['nickname'], 'id' : id }
    userdata['password'] = crypt.crypt(userdata['username'], settings.SECRET_KEY)

    return login_or_register(request, userdata)

