#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django import template
from django.template import Context, loader, RequestContext
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, render_to_response

from moapps.news.models import NewsItem
from moapps.events.models import Event, OutsideEvent
import datetime

def index(request):
    news = NewsItem._default_manager.all()
    news = news.filter( published=True )
    news = news.order_by('created')
    events = Event._default_manager.all()
    events = events.filter( canceled=False )
    events = events.filter( plandate__gt=datetime.date.today() )
    events = events.order_by('plandate')
    outside = OutsideEvent._default_manager.all()
    outside = outside.filter( plandate__gt=datetime.date.today() )
    outside = outside.order_by('plandate')
    c = {
        'news'    : news[:5],
        'events'  : events[:5],
        'outside' : outside[:5],
    }
    return render_to_response('index.html', c, context_instance=template.RequestContext(request))


def support(request):
    return render_to_response('support.html', {}, context_instance=template.RequestContext(request))

def contact(request):
    return render_to_response('contact.html', {}, context_instance=template.RequestContext(request))

def joining(request):
    return render_to_response('joining.html', {}, context_instance=template.RequestContext(request))


