#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
from moapps.news.models import NewsItem
from moapps.contacts.models import Address
import os


class EventLocation(models.Model):
    building = models.CharField('Building Name', max_length=100)
    number   = models.CharField('House Number', max_length=10)
    road     = models.CharField('Street/Road', max_length=255)
    city     = models.CharField('City/Town', max_length=255)
    state    = models.CharField('State', max_length=2, default='MA')
    postcode = models.CharField('Zipcode', max_length=20)
    map      = models.URLField('Map Link', blank=True, null=True)
    website  = models.URLField('Website Link', blank=True, null=True)

    def __unicode__(self):
        return self.building


class Event(models.Model):
    name        = models.CharField(max_length=200)
    description = models.TextField()
    organiser   = models.ForeignKey(User)
    canceled    = models.BooleanField()
    location    = models.ForeignKey(EventLocation, null=True, blank=True)
    plandate    = models.DateField(null=True, blank=True)
    start_time  = models.TimeField(null=True, blank=True)
    end_time    = models.TimeField(null=True, blank=True)
    press       = models.ForeignKey(NewsItem, null=True, blank=True)

    def desclist(self):
        return self.description.split("\r\n\r\n")

    def __unicode__(self):
        return self.name


class OutsideEvent(models.Model):
    name        = models.CharField(max_length=200)
    link        = models.URLField()
    plandate    = models.DateField(null=True, blank=True)
    start_time  = models.TimeField(null=True, blank=True)
    end_time    = models.TimeField(null=True, blank=True)
    location    = models.ForeignKey(EventLocation)
    description = models.TextField()

    def desclist(self):
        return self.description.split("\r\n\r\n")

    def __unicode__(self):
        return self.name

