#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.template import Context, loader, RequestContext
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.utils.simplejson import JSONEncoder
import sys
from moapps.extra.context import contextResponse

def jsonResponse(js):
	return HttpResponse(js, mimetype='application/javascript')

def jsonFunction(method, *args):
	jargs = []
	for datum in args:
		jargs.append( JSONEncoder().encode(datum) )
	return "%s(%s);\n" % (method, ', '.join(jargs))


def jsonContextResponse(request,contextid):
	"""
	The JSON Context Response creates a javascript response which calls
	context methods to help users filling in forms. It uses a similar idea
	to the modelchoice form fields but based on ajax it's able to update
	the list and run filters based on other selected fields in the form.
	"""
	variable = request.REQUEST
	tagvalue = variable.has_key('value') and variable['value']
	taglist  = variable.has_key('list') and variable['list']
	tagid    = variable.has_key('id') and variable['id']

	try:
		plist = contextResponse( contextid, tagvalue, variable, unique=True )
	except NameError, inst:
		type = str(inst.args[0])
		if type == 'filter_failed':
			return jsonResponse(jsonFunction('context_failed_filter', tagid))
		elif type == 'match_failed':
			return jsonResponse(jsonFunction('context_failed_match', tagid))
		return jsonResponse(jsonFunction('context_failed', tagid))

	if len(plist) > 1 or taglist:
		# This means multiple values, different names, a choice!
		olist = []
		for name in plist.keys():
			olist.append( { 'name' : name, 'id' : ','.join(plist[name]) } )
		return jsonResponse(jsonFunction('context_list', olist, tagid))
	elif len(plist) == 1:
		# This means we have a single suggestion, possible multiple values
		name = plist.keys()[0]
		return jsonResponse(jsonFunction('context_match', name, ','.join(plist[name]), tagid))

	raise NameError, "Here be dragons."


