#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#from django.template import Context, loader, RequestContext
#from django.http import HttpResponse, HttpResponseNotFound, HttpResponseRedirect
#from django.shortcuts import get_object_or_404, render_to_response
import sys

contexts = {}


def registerContext(id, model, field_name='name', item_id='id', filter_map={}):
	if contexts.has_key(id):
		sys.stderr.write("Unable to register %s context twice." % id)
	contexts[id] = {
		'model'  : model,
		'field'  : field_name,
		'itemid' : item_id,
		'filter' : filter_map,
	}


def contextResponse( id, value, data, exact_match=False, unique=False ):
	context = contexts[id]
	filters = contextFilters( context['filter'], data )
	field   = context['field']
	itemid  = context['itemid']

	plist = contextList( context, value, filters, exact_match )

	if unique:
		# This will make the id more fuzzy reporting more than
		# One id that this context could be. this format is different
		# As it throws out the item object.
		tlist = {}
		for item in plist:
			if field == 'id':
				name = unicode(item)
			else:
				name = unicode(getattr(item, field))
			if name:
				if tlist.has_key(name):
					tlist[name].append( str(item.id) )
				else:
					tlist[name] = [ str(item.id) ]
		return tlist
	return plist


def contextList( context, value, filters={}, exact_match=False ):
	field   = context['field']
	model   = context['model']   

	if value and len(value) > 1 and id:
		if not exact_match:
			# Do a lookup for anything that either starts with the value
			filters[field+"__istartswith"] = value
		else:
			filters[field+"__iexact"] = value
		
	plist = model._default_manager.filter( **filters ).extra(order_by = [field])[:40]
	if plist:
		return plist

	raise NameError, 'failed_match'


def contextFilters(fs, data):
	filters  = {}
	for inid,x in fs.iteritems():
		# Control the type conversion as well
		# As the filter destination field key.
		itype = str
		if type(x) == list:
			outid = x[0]
			itype  = x[1]
		else:
			outid = x

		if data.has_key(inid) and data[inid] != None:
			try:
				# One of the things we're checking for is a list
				# Of Keys which is used to select multiple objects.
				if itype==int and ',' in data[inid]:
					rs     = data[inid].split(',')
					result = []
					for r in rs:
						result.append( itype( r ) )
					filters[outid+'__in'] = result
				else:
					# This section is an exact match for this field
					filters[outid] = itype(data[inid])
				continue
			except:
				sys.stderr.write("Unable to convert filter %s\n" % inid)
				pass
		else:
			sys.stderr.write("Unable to find filter %s\n" % inid)
		raise NameError, 'failed_filter'
	return filters
