#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.utils.encoding import force_unicode
from django.conf import settings
from django import newforms as forms
from datetime import datetime, time, date
from django.utils.safestring import mark_safe
from context import registerContext, contextResponse

# DATETIMEWIDGET
calbtn = u"""<img src="%simages/calbutton.png" alt="calendar" id="%s_btn" style="cursor: pointer;" title="Select date and time"
			onmouseover="this.style.background='#444444';" onmouseout="this.style.background=''" />
<script type="text/javascript">
	Calendar.setup({
		inputField     :    "%s",
		ifFormat       :    "%s",
		button         :    "%s_btn",
		singleClick    :    true,
		showsTime      :    true
	});
</script>"""


class DateTimeWidget(forms.widgets.TextInput):
	dformat = '%Y-%m-%d %H:%M'
	def render(self, name, value, attrs=None):
		if value is None: value = ''
		final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
		if value != '': 
			try:
				final_attrs['value'] = force_unicode(value.strftime(self.dformat))
			except:
				final_attrs['value'] = force_unicode(value)
		if not final_attrs.has_key('id'):
			final_attrs['id'] = u'%s_id' % (name)
		id = final_attrs['id']
		
		jsdformat = self.dformat #.replace('%', '%%')
		cal = calbtn % (settings.MEDIA_URL, id, id, jsdformat, id)
		a = u'<input%s />%s' % (forms.util.flatatt(final_attrs), cal)
		return mark_safe(a)


	def value_from_datadict(self, data, files, name):
		dtf = forms.fields.DEFAULT_DATETIME_INPUT_FORMATS
		empty_values = forms.fields.EMPTY_VALUES

		value = data.get(name, None)
		if value in empty_values:
			return None
		if isinstance(value, datetime):
			return value
		if isinstance(value, date):
			return datetime(value.year, value.month, value.day)
		for format in dtf:
			try:
				return datetime(*datetime.strptime(value, format).timetuple()[:6])
			except ValueError:
				continue
		return None



class ForeignKeyWidget(forms.widgets.Select):
	def __init__(self, *args, **attrs):
		self.link = attrs.pop('link')
		return super(ForeignKeyWidget, self).__init__(*args, **attrs)

	def render(self, name, value, attrs=None, choices=()):
		drop = super(ForeignKeyWidget, self).render(name, value, attrs, choices)
		drop += self.renderButton(self.link, attrs['id'])
		return drop

	def renderButton(self, href, id):
		result = '<a onclick="return showAddAnotherPopup(this);" id="add_%s" href="%s"><img src="/admin-media/img/admin/icon_addlink.gif" alt="add" style="cursor: pointer;" title="Add New" /></a>' % (id, href)
		return mark_safe(result)


class EmptySet:
	def all(self):
		return []


class ContextChoiceWidget(forms.widgets.Select):
	"""
	Creates a widget which allows the user to select items based
	on a filter, this filter is updated via ajax.
	"""
	def __init__(self, *args, **attrs):
		self.link = attrs.pop('link')
		filters   = {}
		name      = 'name'
		if attrs.has_key('filters'):
			filters = attrs.pop('filters')
		if attrs.has_key('field_name'):
			name = attrs.pop('field_name')
		self.filters = filters.keys()
		registerContext( id=self.link, model=attrs.pop('model'), filter_map=filters, field_name=name, item_id=id )
		return super(ContextChoiceWidget, self).__init__(*args, **attrs)

	def render(self, name, value, attrs=None):
		drop    = ''
		id      = attrs['id']
		filters = ''
		if self.filters:
			filters += "'"
			filters += "', '".join(self.filters)
			filters += "'"
		#attrs['autocomplete'] = "off"
		attrs['onfocus'] = "context(event, [%s], '%s');" % (filters, self.link)
		drop += super(ContextChoiceWidget, self).render(name, value, attrs)
		drop += """
<script id='%s_init'>
  var target = document.getElementById("%s");
  target.focus()
  document.getElementById("%s_init").remove();
</script>
""" % (name, id, name)

#		drop += """
#<script id='%s_init'>
#  initContextList('%s', function {
#	var index = document.getElementById('%s
#    var e = document.createElement( 'div' );
#	e.setAttribute('id', '%s_item_');
#	
#  } );
#</script>""" % (name, id, name, name)

		return mark_safe(drop)



class ContextTextWidget(forms.widgets.TextInput):
	"""
	Creates a widget which will enable the text box to be filled with suggestions
	Using the context javascript ajax code. Otherwise is no different.
	"""
	def __init__(self, *args, **attrs):
		self.link = attrs.pop('link')
		filters   = {}
		name      = 'name'
		id        = 'id'
		if attrs.has_key('filters'):
			filters = attrs.pop('filters')
		if attrs.has_key('field_name'):
			name = attrs.pop('field_name')
		if attrs.has_key('item_id'):
			id = attrs.pop('item_id')

		self.filters = filters.keys()
		registerContext( id=self.link, model=attrs.pop('model'), filter_map=filters, field_name=name, item_id=id )
		return super(ContextTextWidget, self).__init__(*args, **attrs)


	def render(self, name, value, attrs=None):
		drop    = ''
		id      = attrs['id']
		filters = ''
		if self.filters:
			filters += "'"
			filters += "', '".join(self.filters)
			filters += "'"
		attrs['autocomplete'] = "off"
		attrs['onfocus'] = "context(event, [%s], '%s');" % (filters, self.link)
		drop += super(ContextTextWidget, self).render(name, value, attrs)
		drop += """
<script id='%s_init'>
  var target = document.getElementById("%s");
  target.focus()
  document.getElementById("%s_init").remove();
</script>
""" % (name, id, name)
		return mark_safe(drop)

