#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django import newforms as forms
from models import Marketing, MarketingMedia
from moapps.extra.widgets import DateTimeWidget


class FileField(forms.FileField): 
	def __init__(self, *args, **kwargs): 
		super(FileField, self).__init__(*args, **kwargs)
		self.file_exts  = ()
		self.file_types = ()

	def clean(self, data): 
		f = super(FileField, self).clean(data) 
		if not splitext(f.filename)[1].lower() in self.file_exts and not data['content-type'] in self.file_types: 
			raise forms.ValidationError(_(u'Accepted Files: %s' % ' '.join(file_exts))) 
		return f


class MediaField(FileField):
	def __init__(self, *args, **kwargs):
		super(MediaField, self).__init__(*args, **kwargs)
		self.file_exts = ( 'pdf' )
		self.file_types = ( 'application/pdf' )


class SourceField(FileField):
	def __init__(self, *args, **kwargs):
		super(SourceField, self).__init__(*args, **kwargs)
		self.file_exts = ( 'svg' )
		self.file_types = ( 'image/svg+xml', 'image/svg' )


class MarketingMediaForm(forms.ModelForm):
	media  = MediaField()
	source = SourceField()
	class Meta:
		model = MarketingMedia


class MarketingForm(forms.ModelForm):
	class Meta:
		model   = Marketing
		exclude = ('organiser','project_ptr')

