# -*- coding: utf-8 -*- 
#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.db import models
from django.contrib.auth.models import User
from django.conf import settings
import os


class Project(models.Model):
    name        = models.CharField(max_length=200)
    description = models.TextField()
    organiser   = models.ForeignKey(User, related_name='project_owner')

    def __unicode__(self):
        return self.name

    def root_tasks(self):
        return self.projecttask_set.filter( parent = None )

    def can_modify(self, user, parent_task=None):
        if self.organiser.id == user.id:
            return True
        return False

    def desclist(self):
        return self.description.split("\r\n\r\n")

    class Admin:
        pass



class ProjectTask(models.Model):
    project  = models.ForeignKey(Project)
    parent   = models.ForeignKey("self", null=True, blank=True)
    name     = models.CharField(max_length=200)
    owner    = models.ForeignKey(User, related_name='project_task_owner', blank=True, null=True)
    done     = models.BooleanField(default=False)
    active   = models.BooleanField(default=False)
    duedate  = models.DateTimeField('Due Date', null=True, blank=True)
    helpneed = models.IntegerField(default=0, blank=True, null=True)
    helpers  = models.ManyToManyField(User, null=True, blank=True, related_name='helper')
    linkurl  = models.URLField(null=True, blank=True)

    def __unicode__(self):
        return self.name

    def isdone(self):
        # Could calculate doneness from child tasks too    
        return self.done

    def isactive(self):
        return self.active

    def can_modify(self, user):
        if (self.owner and self.owner.id == user.id) or self.project.can_modify(user):
            return True
        return False

    class Admin:
        pass





class ProjectTemplate(models.Model):
    name        = models.CharField(max_length=200)
    description = models.TextField()
    classtype   = models.CharField(max_length=100)

    def __unicode__(self):
        return self.name

    def cloneTasks(self, event):
        """
        Take each of the tasks from the event template tasks and record them into the event.
        """
        pass

    class Admin:
        pass


class ProjectTemplateTask(models.Model):
    template = models.ForeignKey(ProjectTemplate)
    parent   = models.OneToOneField("self", null=True, blank=True)
    name     = models.CharField(max_length=200)
    helpneed = models.IntegerField()
    linkurl  = models.URLField()
    duespan  = models.CharField(max_length=50)

    def __unicode__(self):
        return self.name


