#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django import template
from django.template import Context, loader, RequestContext
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.views.generic.list_detail import object_list
from django.utils.simplejson import JSONEncoder

from models import Project, ProjectTask
from moapps.projects import project_type
from moapps.extra.ajax import jsonResponse, jsonFunction

import sys


def project_types():
	result = []
	for name,value in project_type.iteritems():
		result.append( {
			'name' : name,
			'id'   : name.lower().replace(' ',''),
			'description' : value['description'],
		} )
	return result
		

def index(request):
	queryset = Project._default_manager.all().filter( )
	return object_list(request, queryset=queryset, template_name='projects/index.html' )


def create(request):
	rc = template.RequestContext(request)
	typeid = None
	if request.GET.has_key('type'):
		typeid = request.GET['type']

	if not typeid or not project_type.has_key(typeid):
		return render_to_response('projects/create-type.html', { 'types' : project_types() }, context_instance=RequestContext(request))
	type = project_type[typeid]
	form_class = type['form']

	if request.method == 'POST':
		form = form_class(data=request.POST)
		if form.is_valid():
			new_project = form.save(commit=False)
			new_project.organiser = rc['user']
			new_project.save()
			return HttpResponseRedirect('/projects/%s/' % new_project.id)
	else:
		form = form_class()

	return render_to_response('projects/create-%s.html' % typeid.lower(), { 'form': form, 'type': typeid }, context_instance=RequestContext(request))


def view(request, project_id):
	projectitem = get_object_or_404(Project, id=project_id)
	return render_to_response('projects/item.html', { 'project': projectitem }, context_instance=template.RequestContext(request))


def edit(request, project_id):
	rc = template.RequestContext(request)
	user = rc['user']
	project = get_object_or_404(Project, id=project_id)
	if project.organiser == user:
		if request.method == 'POST':
			form = None #ProjectForm(data=request.POST, files=request.FILES, instance=project)
			if form.is_valid():
				form.save()
				return HttpResponseRedirect('/projects/%s/' % project.id)
		else:
			form = None #ProjectForm(instance=project)

		return render_to_response('projects/edit.html', { 'form': form }, context_instance=RequestContext(request))
	else:
		return HttpResponseNotFound()


# ============= JSON - AJAX =============== #
#
# What follows is requests that are strictly
# ajax only request.
#
# ========================================= #

def add_item(request, project_id):
	"""
	Create a new task item for a project, giveing it a specific
	parent, return the new object as JSON.
	"""
	rc = template.RequestContext(request)
	project = get_object_or_404(Project, id=project_id)
	parent  = None
	if request.GET.has_key('parent'):
		parent = get_object_or_404(ProjectTask, id=request.GET['parent'], project=project.id)
	if project.can_modify(rc['user']) or (parent and parent.can_modify(rc['user'])):
		task = ProjectTask(
			project = project,
			name    = 'New Project Task',
			done    = (parent and parent.done) or 0,
			owner   = (parent and parent.owner),
			parent  = parent,
		)
		task.save()
		return jsonResponse(task_jsoned('item_added', task))
	# Add a new item and return some javascript
	return HttpResponseNotFound('Unable to Create')


def edit_item(request, project_id=None):
	"""
	Edit a task name.
	"""
	rc      = template.RequestContext(request)
	user    = rc['user']
	task_id = request.GET['id']
	project = get_object_or_404(Project, id=project_id)
	task    = get_object_or_404(ProjectTask, id=task_id, project=project.id)
	
	if task.can_modify(user):
		if request.GET.has_key('name'):
			print "Setting name to %s" % request.GET['name']
			task.name = request.GET['name']
		task.save()
		return jsonResponse(task_jsoned('item_edited', task))
	return HttpResponseNotFound('Unable to edit')
		

def delete_item(request, project_id=None):
	"""
	Delete a task from a project and return deleted True as JSON
	"""
	rc   = template.RequestContext(request)
	user = rc['user']
	task_id = request.GET['id']
	project = get_object_or_404(Project, id=project_id)
	task    = get_object_or_404(ProjectTask, id=task_id, project=project.id)

	if task.can_modify(user):
		task.delete()
		return jsonResponse(jsonFunction('item_deleted', { 'deleted' : True, 'id' : task_id }))
	return HttpResponseNotFound('Unable to delete')


def mark_item(request, project_id=None):
	"""
	Mark a task as done or not done and return the task data as JSON
	"""
	rc      = template.RequestContext(request)
	user    = rc['user']
	project = get_object_or_404(Project, id=project_id)
	task    = get_object_or_404(ProjectTask, id=request.GET['id'], project=project.id)
	if task.can_modify(user):
		if request.GET['done'] == '1':
			task.done = True
			if not task.owner:
				task.owner = user
		else:
			task.done = False
		task.save()
		return jsonResponse(task_jsoned('replace_item', task))
	return HttpResponseNotFound('Unable to Mark Task')


def take_item(request, project_id=None):
	"""
	Take ownership of this task and return task data as JSON
	"""
	rc      = template.RequestContext(request)
	user    = rc['user']
	project = get_object_or_404(Project, id=project_id)
	task    = get_object_or_404(ProjectTask, id=request.GET['id'], project=project.id)
	if user:
		task.owner = user
		task.save()
		return jsonResponse(task_jsoned('replace_item', task))
	return HttpResponseNotFound('Cant claim task, not logged in')


def give_item(request, project_id=None):
	"""
	Give up ownership of this task and return task data as JSON.
	"""
	rc      = template.RequestContext(request)
	user    = rc['user']
	project = get_object_or_404(Project, id=project_id)
	task    = get_object_or_404(ProjectTask, id=request.GET['id'], project=project.id)
	if task.can_modify(user):
		task.owner = None
		task.save()
		return jsonResponse(task_jsoned('replace_item', task))
	return HttpResponseNotFound('Unable to give up item')


def task_jsoned(method, task):
	return jsonFunction( method, {
		'id'      : task.id,
		'name'    : task.name,
		'done'    : task.isdone(),
		'active'  : task.isactive() and '0' or '1',
		'project' : task.project.id,
		'parent'  : task.parent and task.parent.id,
		'owner'   : task.owner and task.owner.id,
		'oname'   : task.owner and "%s %s" % (task.owner.first_name, task.owner.last_name),
		'error'   : False,
	} )


