#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.db import models
from django.contrib.auth.models import User

META_CHOICES = (
	( 'SOFT', 'Software' ),
	( 'SERV', 'Service' ),
	( 'COMM', 'Community' ),
)

ITEM_CHOICES = (
	( 'PROD', 'Physical Product' ),
	( 'META', 'Meta Product' ),
	( 'TAX',  'Tax Item' ),
	( 'SHIP', 'Shipping Item' ),
)

class ProductBase(models.Model):
	sku         = models.CharField(max_length=100)
	name        = models.CharField(max_length=200)
	description = models.TextField(null=True, blank=True)
	published   = models.BooleanField(default=False)
	
	
	def __unicode__(self):
		return self.nam



class PhysicalProduct(ProductBase):
	# Physical Metrics
	weight      = models.FloatField(default=0)
	length      = models.FloatField(default=0)
	width       = models.FloatField(default=0)
	height      = models.FloatField(default=0)

	# Stocking Options
	preorders   = models.BooleanField(default=False)
	stocklevel  = models.IntegerField(default=0)
	restockdate = models.DateTimeField(null=True, blank=True)

	class Admin:
		pass



class MetaProduct(ProductBase):
	type    = models.CharField(max_length=4, choices=META_CHOICES)
	linkurl = models.URLField(blank=True, null=True)

	class Admin:
		pass



class Order(models.Model):
	orderer = models.ForeignKey(User)
	ordered = models.DateTimeField("Ordered Date", auto_now_add=True)

	def __unicode__(self):
		return "Order %d by %s on %s" % (self.id, str(self.orderer), str(self.ordered))

	class Admin:
		pass


class OrderItem(models.Model):
	order   = models.ForeignKey(Order)
	cost    = models.IntegerField()
	type    = models.CharField(max_length=4, choices=ITEM_CHOICES)
	product = models.ForeignKey(ProductBase, null=True, blank=True)



