#
# Copyright 2008, Martin Owens.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django import template
from django.template import Context, loader, RequestContext
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
import workflow, blocks
import sys, os


def index_index(request):
	context = template.RequestContext(request)
	data    = { 'data': blocks.blocks.values() }
	return render_to_response('workflows/index.html', data, context_instance=context)


def index_view(request, object_class):
	context = template.RequestContext(request)
	b = blocks.getBlocks( object_class )
	if b:
		data = { 'data': b.index.list(), 'name': b, 'description': '', 'type':object_class }
		return render_to_response('workflows/index_view.html', data, context_instance=context)
	raise Http404


#def viewstep(request, workflow_id, step=0):
#	context = template.RequestContext(request)
#	sys.stderr.write("Trying to get workflow %s / %s\n" % (workflow,str(step)))
#	if workflow_id and workflow.flows.has_key(workflow_id):
#		flow = workflow.flows[workflow_id]
#		(form,name,desc) = flow
#		if form:
#			return render_to_response('workflows/generic.html', { 'form': form(prefix='core'), 'name': name, 'description': desc }, context_instance=context)
#	raise Http404


def view_view(request, object_class, object_id):
	context = template.RequestContext(request)
	b = blocks.getBlocks( object_class )
	if b:
		obj = get_object_or_404(b.model, id=object_id)
		view_block = b.view( obj )
		return render_to_response('workflows/view.html', { 'content': view_block.view(), 'type': b, 'object':obj }, context_instance=context )
	raise Http404


def edit_view(request, object_class, object_id=None):
	context = template.RequestContext(request)
	b = blocks.getBlocks( object_class )
	if b:
		obj = get_object_or_404(b.model, id=object_id)
		edit = b.edit( obj )
		if request.method == 'POST':
			if edit.save(request.POST):	
				return HttpResponseRedirect('/workflow/view/%s/%d/' % (b.id, obj.id))
		return render_to_response('workflows/edit.html', { 'form': edit.getForm(), 'type': b, 'object': obj }, context_instance=context )
	raise Http404
	

def create_view(request, object_class):
	return edit_view(request, object_class)


def discover_view(request, object_class):
	context = template.RequestContext(request)
	b = blocks.getBlocks( object_class )
	if b:
		d = b.discovery
		if request.method == 'POST':
			form = d.getForm(data=request.POST)
			ids  = request.POST[d.return_field()]
			if ',' in ids:
				pass # Do something with multiple ids
			return HttpResponseRedirect('/workflow/view/%s/%d/' % (b.id, int(ids)))
		else:
			form = d.getForm()
		return render_to_response('workflows/discovery.html', { 'form': form, 'type': b }, context_instance=context )
	raise Http404





