# -*- coding: utf-8 -*-
import unittest
from moxml.config import Config, Directory, Hash

binary_data = '''AP///////wBMLRUBOTFTRxQPAQNsJh54Km+GolpNlCQaT1S/74CBgHFPAQEBAQEBAQEBAQEBMCoA
mFEAKkAwcBMAeC0RAAAeAAAA/QA4Sx5RDgAKICAgICAgAAAA/ABTeW5jTWFzdGVyCiAgAAAA/wBI
TUVZNTAyOTA3CiAgAE8K\n''';

class TestCase(unittest.TestCase):
	def setUp(self):
		self.manager = Directory('data/')
		self.config = Config( 'test.xml', self.manager )

	def testManager(self):
		assert(self.manager, 'Test data directory')

	def testCreation(self):
		assert(self.config, 'Test xml loaded')

	def testHashByDefault(self):
		assert(type(self.config)==Hash, 'Hash type by default')

	def testString(self):
		assert(self.config['value1']=='Blah', 'Get string value from Hash')

	def testNumber(self):
		assert(self.config['value2']==4443, 'Get number value from Hash')

	def testBinary(self):
		assert(str(self.config['bytes'])==binary_data, 'Get binary data from Hash')

	def testArray(self):
		assert(self.config['array']==[ 1, 2, 3 ], 'Get array structure from Hash')

	def testHash(self):
		print self.config['hash']==[]
		assert( self.config['hash']==[], 'Get hash structure from Hash')

	def testValues(self):
		assert(self.config['hash'].values()==12, 'Get hash values')

	def testLanguageDefault(self):
		assert(self.config['langstring']=='english', 'Default language is english')

	def testLanguageFr(self):
		# Set the language here somehow XXX (FR).
		assert(self.config['langstring']=='french', 'Language set to french')

	def testLanguageGb(self):
		# Set the language here somehow XXX (en_GB).
		assert(self.config['langstring']=='british', 'Language set to british (sub-en)')

	def testLanguageJp(self):
		# Set the language here somehow XXX (JP).
		assert(self.config['langstring']==u'書込み', 'Language set to japanese using utf-8 characters')

	def testLanguageNot(self):
		# Set the language here somehow XXX (DE).
		assert(self.config['langstring']=='english', 'Language that isnt available so fallback')

	def testLanguageSpecific(self):
		assert(self.config['langstring'].language('fr')=='french', 'Ask for language specifically')

	def testLanguageSpecificNot(self):
		assert(self.config['langstring'].language('fr_CA')=='french', 'Ask for language specifically and fallback')

	def testSetLanguage(self):
		# Set the language here somehow XXX (DE).
		self.config['langstring'].set('german')
		assert(self.config['langstring'].language('fr')=='french', 'Setting a language value doesnt kill existing data.')
		assert(self.config['langstring']=='german', 'Set language based value')
		self.config['langstring'].delLanguage('de');



suite = unittest.makeSuite(TestCase,'test')
