/*
 * Copyright 2008, Martin Owens.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var items;
var root;
var count = 0;
var editing = null;
var edittype = null;

function init() {
	items = document.getElementById('agenda');
	root  = document.getElementById('item0');
	if(! root) {
		init_root()
	} else {
		count = root.getAttribute('count');
	}
}

function init_root() {
	root = create_item( items, 'All Agenda Items', 1 )
}

// additem( parent_id )
// Creates a new agenda item as a child of the parent_id (default = 0)
function add_item(p) {
	p = document.getElementById('item'+p);
	create_item( p, "New Child Element", 0 );
	edit_item( count, 'name' );
}

function create_item( pe, txt, root ) {
	count = count + 1;

	var cse = document.createElement('div');
	cse.setAttribute('id', 'item' + count);
	cse.setAttribute('class', 'case');

	var item = document.createElement('div');
	item.setAttribute('class', 'item');

	if(root) {
		item_image(item, 'start', null, null);
	} else {
		item_image(item, 'del', 'Delete Item', 'delete_item('+count+')');
	}
	item_image(item, 'add', 'Add Child Item', 'add_item('+count+')');

	var text = document.createElement('div');
	text.setAttribute('class', 'text');
	text.setAttribute('id', 'text'+count);
	text.appendChild(document.createTextNode(txt));
	item.appendChild(text);

	item_image(item, 'end', null, null );
	if(! root) {
		item_image(item, 'link', 'Edit Item Link', 'edit_item('+count+', "link")');
		item_image(item, 'name', 'Edit Item Text', 'edit_item('+count+', "name")');
	}

	cse.appendChild(item);
	pe.appendChild(cse);
	return cse;
}

function item_image(p, cls, title, click) {
	var e = document.createElement('img');
	e.setAttribute('src', '/media/images/agenda/'+cls+'.png');
	e.setAttribute('class', cls);
	if(title) {
		e.setAttribute('title', title);
	}
	if(click) {
		e.setAttribute('onclick', click);
	}
	p.appendChild(e);
}

function delete_item( id ) {
	var item = document.getElementById('item'+id);
	if(item) {
		item.parentNode.removeChild(item);
	}
}

function edit_item( id, type ) {
	var text = document.getElementById('text'+id);
	editing = id;
	edittype = type;

	// Remove text from block
	if(text.firstChild.nodeType==text.TEXT_NODE) {
		str  = text.firstChild.nodeValue;
		link = null;
	} else {
		link = text.firstChild.getAttribute('href');
		str  = text.firstChild.firstChild.nodeValue;
	}

	text.removeChild(text.firstChild);

	// Create input box replacement
	var input = document.createElement('input');
	input.setAttribute('class', 'editor');
	if(type=='link') {
		input.setAttribute('value', link);
		input.setAttribute('str', str);
	} else {
		input.setAttribute('value', str);
		input.setAttribute('link', link);
	}
	input.setAttribute('onkeypress', 'edit_keyed(event)');
	input.setAttribute('onblur', 'finish_edit()');
	text.appendChild(input);
	input.focus();
	input.setSelectionRange(0,str.length);
}

function edit_keyed(event) {
	var keyCode = event.keyCode;
	if(editing != null) {
		if(keyCode==13) {
			finish_edit();
		}
	}
}

function finish_edit() {
	if(editing != null) {
		var text = document.getElementById('text'+editing);
		editing = null;
		if(text) {
			if(edittype=='link') {
				link = text.firstChild.value;
				str = text.firstChild.getAttribute('str');
			} else {
				str = text.firstChild.value;
				link = text.firstChild.getAttribute('link');
			}
			text.removeChild(text.firstChild);
			if(link) {
				var lo = document.createElement('a');
				lo.setAttribute('href', link);
				lo.setAttribute('target', '_blank');
				lo.appendChild(document.createTextNode(str));
				text.appendChild(lo);
			} else {
				text.appendChild(document.createTextNode(str));
			}
		}
		edittype = null;
	}
}

function debug(item) {
	var result = '';
	for(i in item) {
		if(item[i]) {
			result = result + i + '="' + item[i] + '", ';
		} else {
			result = result + i + ', ';
		}
		if(result.length > 400) {
			alert(result);
			result = '';
		}
	}
	if(result) {
		alert(result);
	}
}


