/* 
 * Copyright 2008, Martin Owens.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/*

The attributes and methods required to do async javascript access.

Method - Calling methods with data which return data in the form of js method calls.
  ajax_request( method_name, data_hash );
  ajax_return( data );

Poll - Calling a polling request every so often for updates, returns javascript method calls to perform actions.
  ajax_start_poll( poll_name, poll_method, data_hash, timeout );
  ajax_stop_poll( poll_name );

*/

var polls = {};

function ajax_request( method_name, data_hash, direct ) {
	url = 'ajax/' + method_name + '/'
	if(direct) {
		url = method_name
	}
	// Call method on server with data, pass ajax_return as return method.
	new Ajax.Request( url,
		{ method:'get', onSuccess: ajax_return, onFailure: ajax_error, parameters: data_hash }
	);
}

function ajax_return( transport ) {
	// This is called when the server gets back to us about data.
	// We need to execute each of the methods that the server thinks needs to be run.
	var response = transport.responseText || "alert('No Response from server!');";
	var result = eval(response);
	return result;
}

function ajax_error( transport ) {
	document.write(transport.responseText);
}

/*function ajax_start_poll( poll_name, poll_method, data_hash, timeout ) {
	polls[poll_name] = [ poll_method, data_hash, timeout ];
	ajax_do_poll( poll_name );
}

function ajax_stop_poll( poll_name ) {
	polls[poll_name] = null;
}

function ajax_do_poll( poll_name ) {
	poll = polls[poll_name];
	method  = poll[0];
	data    = poll[1];
	timeout = poll[2];
	ajax_request(method, data);
	document.setTimeout('ajax_do_poll("'+poll_name+'")', timeout);
}*/


function debug(item) {
	var result = '';
	for(i in item) {
		if(item[i]) {
			result = result + i + '="' + item[i] + '", ';
		} else {
			result = result + i + ', ';
		}
		if(result.length > 400) {
			alert(result);
			result = '';
		}
	}
	if(result) {
		alert(result);
	}
}

var contexts = new Array();
var rcontext = new Array();

function context( event, filters, get_method ) {
	var tag = event.target
	var id  = tag.id
	if(!contexts[id]) {
		if(tag && tag.tagName == 'INPUT') {
			tag.setAttribute('onkeydown', "context_keydown(event);");
			tag.setAttribute('onkeyup', "context_keyup(event);");
			contexts[id] = [ filters, get_method ];

			for(var i=0;i<filters.length;i++) {
				var ftag = document.getElementById(filters[i]);
				if(ftag && ftag.getAttribute('type') != 'hidden') {
					// Try and tie the filter tag back to the updating of this field.
					ftag.setAttribute('onchange', 'context_update("'+id+'");');
				} else {
					if(!rcontext[filters[i]]) {
						rcontext[filters[i]] = new Array();
					}
					rcontext[filters[i]].push(id);
				}
			}
			if(tag.value) {
				context_update(id);
			}
		}
	}
}

function context_keyup( event ) {
	var target = event.target
	var id     = target.id
	var value  = target.value
	var key    = event.keyCode;

	if(key == 40) {
		return context_list_select(id, 'next');
	} else if(key == 39) {
		return context_list_select(id, 'prev');
	}

	// This skipping code is here so when the user is typing in
	// the word as it appears in the existing suggestion we don't
	// need to send another request for it to the server.
	if(target.hasAttribute('skip') && target.getAttribute('skip')!='!') {
		var end = target.getAttribute('skip');
		var org = target.value;
		target.value = org + end;
		target.selectionStart = org.length
		target.selectionEnd = org.length + end.length;
		target.setAttribute('skip','!');
		return;
	}
	if(value.length > 2) {
		return context_update(id);
	}
	return context_clean( id );
}

function context_update( id ) {
	var target     = document.getElementById( id );
	var value      = target.value;
	var filters    = contexts[id][0];
	var get_method = contexts[id][1];
	if(get_method && value != '') {
		var data = { 'id' : id, 'value' : value };
		for(var i=0;i<filters.length;i++) {
			if(document.getElementById(filters[i])) {
				data[filters[i]] = document.getElementById(filters[i]).value;
			} else {
				return context_new_value( id );
			}
		}
		return ajax_request( "/extra/context/"+get_method+"/", data, true );
	}
}

function context_keydown( event ) {
	var target  = event.target;
	var keycode = event.keyCode;
	var letter  = String.fromCharCode(keycode);
	var value   = target.value;
	if(keycode < 9 || keycode > 27) {
		// Check to see if the letter typed is the letter that
		// will be replaced by the key stroke action. Tell the
		// updater to skip calling the server if so.
		if(target.selectionStart != target.selectionEnd) {
			var next = value.substring(target.selectionStart,target.selectionStart+1)
			var end  = value.substring(target.selectionStart+1,value.length)
			if(next.toLowerCase()==letter.toLowerCase()) {
				target.setAttribute('skip',end);
				return;
			}	
			var text = target.value;
			target.value = text.substring(0,target.selectionStart);
		}
	}
}

function context_create_hidden( tagid ) {
	var result = document.createElement( 'input' );
	result.setAttribute('type','hidden');
	result.setAttribute('name', tagid);
	result.setAttribute('id', tagid);
	return result;
}

function context_clean( tagid ) {
	var input  = document.getElementById( tagid );
	input.setAttribute('class','');
	var hidden = document.getElementById( tagid + '_id' );
	if(hidden) { hidden.remove() }
	var list   = document.getElementById( tagid + '_list' );
	if(list) { list.remove() }
}

function context_list( data, tagid ) {
	context_clean( tagid );
	var input  = document.getElementById( tagid );
	var c = document.createElement('div');
	var h = 0;
	var w = input.offsetWidth-2;
	var t,l = findPos(input);
	c.setAttribute('class', 'contextlist');
	c.setAttribute('style', 'width: '+w+'px; height: '+h+'px; top: '+t+'px; left: '+l+'px;');
	c.setAttribute('id', tagid + '_list');
	input.parentNode.appendChild(c);

	for(var i=0;i<data.length;i++) {
		var datum = data[i];
		var entry = document.createElement('div');
		entry.setAttribute('class','contextitem');
		entry.appendChild(document.createTextNode(datum['name']));
		entry.setAttribute('id',tagid+"_list_item"+i);
		entry.setAttribute('index',i);
		entry.setAttribute('onclick', 'context_match("'+datum['name']+'", "'+datum['id']+'", "'+tagid+'", 1);');
		c.appendChild(entry);
		h = h + entry.offsetHeight;
	}
	c.setAttribute('style', 'width: '+w+'px; height: '+h+'px; top: '+t+'px; left: '+l+'px;');
}

function context_list_select( tagid, index ) {
	var list = document.getElementById(tagid+"_list");

	if(index == 'next') {
		index = 0;
		if(list.hasAttribute('selected')) {
			index = list.getAttribute('selected')+1;
		}
	} else if(index == 'prev') {
		index = list.getAttribute('selected')-1;
	}

	var item = document.getElementById(tagid+"_list_item"+index);
	if(list && item) {
		var olditem = document.getElementById(tagid+"_list_item"+list.getAttribute('selected'));
		if(olditem) {
			olditem.setAttribute('class', 'contextitem');
		}
		item.setAttribute('class','contextsitem');
		list.setAttribute('selected', index);
	}
}

function context_match( data, id, tagid, noselect ) {
	var input  = document.getElementById( tagid );
	var exists = input.value

	// For some odd reason some events happen twice, even though they
	// shouldn't, so to cope with this We need to make sure we're not
	// destroying the selection we put in place.
	if(input.selectionStart != input.selectionEnd) {
		noselect = true;
	}

	context_clean( tagid );
	var hidden = context_create_hidden( tagid+'_id' );
	input.parentNode.insertBefore(hidden, input);
	input.value = data;
	hidden.value = id;

	if(!noselect) {
		// There is code to do this in IE using createTextRange
		// But I don't like IE so I won't support it here.
		input.focus();
		input.selectionStart = exists.length;
		input.selectionEnd = data.length;
	}
	input.setAttribute('class','existing');
	context_update_deps( tagid );
}

function context_failed_match(tagid)        { context_failed(tagid); }
function context_failed_filter(tagid, ftag) { context_failed(tagid); }
function context_failed(tagid) {
	context_new_value(tagid);
}

function context_new_value(tagid) {
	var input = document.getElementById(tagid);
	context_clean(tagid);
	if(input.value.length > 1) {
		input.setAttribute('class', 'new');
		context_update_deps( tagid );
	}
}

// This tries to update all contexts which are dependant
// on tagid using a reverse context lookup.
function context_update_deps( tagid ) {
	var r = rcontext[tagid+"_id"];
	if(r) {
		for(var i=0;i<r.length;i++) {
			context_update(r[i]);
		}
	}
}

function findPos(obj) {
	var curleft = curtop = 0;

	if (obj.offsetParent) {
		do {
			curleft += obj.offsetLeft;
			curtop += obj.offsetTop;
		} while (obj = obj.offsetParent);
	}

	return [curleft,curtop];
}



























